import { ComponentLike } from '@glint/template';
import Component from '@glimmer/component';
import { SortConstants } from '../constants/sort-constants';
import { TrackedArray } from 'tracked-built-ins';
import ModelsTableColumn, { ModelsTableColumnOptions } from '../utils/emt/emt-column';
import DefaultTheme from '../services/emt-themes/default';
import { ModelsTableArgs } from '../interfaces/components/models-table-args.interface';
import { ModelsTableSignature } from '../interfaces/components/models-table-signature.interface';
import { ColumnDropdownOptions } from '../interfaces/column-dropdown-options.interface';
import { ColumnSet } from '../interfaces/column-set.interface';
import { SelectOption } from '../interfaces/select-option.interface';
import { ModelsTableDataItem } from '../types/models-table-data-item.type';
import { ColumnComponents } from '../interfaces/column-components.interface';
import { GroupedHeader } from '../interfaces/grouped-header.interface';
import { SortMap } from '../interfaces/sort-map.interface';
import { DataGroupProperty } from '../types/data-group-property.type';
import { ColumnCustomSortFn } from '../types/column-custom-sort-fn.type';
/**
 * Table-component with pagination, sorting and filtering.
 *
 * It should be used when whole dataset is already loaded. For server-side pagination, filtering and sorting
 * {@link ModelsTableServerPaginated} should be used.
 *
 * Basic usage example:
 *
 * ```html
 * <ModelsTable @data={{model}} @columns={{this.columns}} @themeInstance={{this.themeInstance}} />
 * ```
 *
 * Usage with block context:
 *
 * ```html
 * <ModelsTable @data={{this.data}} @columns={{this.columns}} @themeInstance={{this.themeInstance}} as |MT|>
 *   <MT.GlobalFilter />
 *   <MT.DataGroupBySelect />
 *   <MT.ColumnsDropdown />
 *   <MT.Table />
 *   <MT.Footer />
 * </ModelsTable>
 * ```
 *
 * Usage with block content 2:
 *
 * ```html
 * <ModelsTable @data={{this.data}} @columns={{this.columns}} @themeInstance={{this.themeInstance}} as |MT|>
 *   <MT.GlobalFilter />
 *   <MT.DataGroupBySelect />
 *   <MT.ColumnsDropdown />
 *   <MT.Table />
 *   <MT.PaginationNumeric />
 *   <MT.PaginationSimple />
 *   <MT.Summary />
 *   <MT.PageSizeSelect />
 * </ModelsTable>
 * ```
 *
 * ModelsTable yields references to the following contextual components:
 *
 * * {@link DefaultTheme.GlobalFilter | GlobalFilter} - global filter used for table data
 * * {@link DefaultTheme.ColumnsDropdown | ColumnsDropdown} - dropdown with list of options to toggle columns and column-sets visibility
 * * {@link DefaultTheme.DataGroupBySelect | DataGroupBySelect} - dropdown to select property for table-rows grouping
 * * {@link DefaultTheme.Table | Table} - table with a data
 * * {@link DefaultTheme.Footer | Footer} - summary and pagination
 * * {@link DefaultTheme.Summary | Summary} - component with summary info about table data. It also contains button to clear all filters applied to the table
 * * {@link DefaultTheme.PageSizeSelect | PageSizeSelect} - component with a page sizes dropdown. It allows to select number if records shown on page
 * * {@link DefaultTheme.PaginationNumeric | PaginationNumeric} - component with a table navigation. It allows to move to the page by its number
 * * {@link DefaultTheme.PaginationSimple | PaginationSimple} - component with a table navigation. It allows to move to the first, last, prev and next pages (this four buttons are shown always)
 *
 * Check own docs for each component to get detailed info.
 *
 * References to the following properties are yielded:
 *
 * * {@link processedColumns} - list of columns in the table
 * * {@link visibleProcessedColumns} - list of currently visible columns in the table
 * * {@link visibleContent} - list of records shown in the current table's page
 * * {@link groupedHeaders} - extra headers for table's header. DOESN'T work properly with visibility toggle for columns
 * * {@link groupedVisibleContent} - list of records shown in the current table's page when rows grouping is enabled
 * * {@link groupedVisibleContentValuesOrder} - list of values for property used to group rows
 * * {@link groupedArrangedContent} - list of all filtered and sorted records in the table used when rows grouping is enabled
 * * {@link displayGroupedValueAs} - determines how value for grouping is shown. Can "row" and "column"
 * * {@link sortByGroupedFieldDirection} - sorting order for property used to group rows
 * * {@link useDataGrouping} - flag to turn on/off rows grouping
 * * {@link globalFilterUsed} - `true` if global filter is used
 * * {@link filterString | globalFilter} - value of global filter
 * * {@link anyFilterUsed} - `true` when any filter (global or for column) is used. `false` when no filters are used
 * * {@link useFilteringByColumns} - determines if columns filtering should be used. By default if it's `false` header's row with filter input/select fields is not shown
 * * {@link collapsedGroupValues} - list of values for property used to group rows. Row group with this value are collapsed
 * * {@link pagesCount} - number of pages in the table
 * * {@link arrangedContentLength | recordsCount} - number of records in the table
 * * {@link firstIndex} - first row's index in the current page
 * * {@link lastIndex} - last row's index in the current page
 * * {@link currentPageNumber} - current page
 * * {@link currentPageNumberOptions} - list of options for current page
 * * {@link pageSizeOptions} - list of options for page size select
 * * {@link pageSize} - number of rows shown in the table's page
 * * {@link columnDropdownOptions} - list of options for columns dropdown
 * * {@link allColumnsAreHidden} - `true` if all columns are hidden. `false` if at least one column is visible
 * * {@link dataGroupOptions} - list of property names used for rows grouping
 * * {@link currentGroupingPropertyName} - property name used now for grouping rows
 * * {@link themeInstance} - theme instance used in the table
 * * {@link expandedItems} - list of expanded rows
 * * {@link selectedItems} - list of selected rows
 * * {@link sortProperties} - list of data-properties used to sort table rows
 * * {@link Core.ModelsTableServerPaginated.isLoading | isLoading} - `true` when data for table is loading (used only for {@link ModelsTableServerPaginated})
 * * {@link Core.ModelsTableServerPaginated.isError | isError} - `true` when request for data loading failed (used only for {@link ModelsTableServerPaginated})
 *
 * References to the following actions are yielded:
 *
 * * {@link showAllColumns} - show all columns in the table (by default used in the columns dropdown)
 * * {@link hideAllColumns} - hide all column in the table (by default used in the columns dropdown)
 * * {@link restoreDefaultVisibility} - show columns visible initially (by default used in the columns dropdown)
 * * {@link toggleColumnSetVisibility} - toggle visibility for columns in the custom columns set (by default used in the columns dropdown)
 * * {@link toggleColumnVisibility} - toggle visibility for column (by default used in the columns dropdown)
 * * {@link expandRow} - expand or collapse row to show details
 * * {@link collapseRow} - select or deselect row
 * * {@link expandAllRows} - expand all rows
 * * {@link collapseAllRows} - collapse all rows
 * * {@link toggleAllSelection} - select or deselect all rows
 * * {@link goToPage} - change table's page (by default used in the pagination)
 * * {@link clearFilters} - clear all filters (by default used in the summary)
 * * {@link sort} - sort provided column
 * * {@link toggleGroupedRowsSelection} - select or deselect all rows in the group (for cases when rows grouping is used)
 * * {@link toggleGroupedRowsExpands} - expand or collapse all rows in the group (for cases when rows grouping is used)
 * * {@link toggleGroupedRows} - toggle rows group (for cases when rows grouping is used)
 * * {@link clickOnRow}
 * * {@link doubleClickOnRow}
 * * {@link hoverOnRow}
 * * {@link outRow}
 * * {@link changePageSize} - set number of rows on the page
 * * {@link changeGlobalFilter} - change global filter value
 * * {@link changeColumnFilter} - change filter value for column
 * * {@link changeGroupingPropertyName} - change property used to group rows
 *
 * ModelsTable has a lot of options you may configure, but there are two required properties called `data` and `columns`.
 * First one contains data (e.g. list of records from the store).
 * Second one is a list of table's columns (check {@link Core.ModelsTableColumnOptions | ModelsTableColumnOptions} for available options).
 *
 * @class ModelsTable
 */
export default class ModelsTableComponent<T> extends Component<ModelsTableSignature<T>> {
    /**
     * True if data is currently being loaded from the server.
     * Can be used in the template to e.g. display a loading spinner.
     *
     * Used only for {@link ModelsTableServerPaginated}
     */
    isLoading: boolean;
    /**
     * True if last data query promise has been rejected.
     * Can be used in the template to e.g. indicate stale data or to e.g. show error state.
     *
     * Used only for {@link ModelsTableServerPaginated}
     */
    isError: boolean;
    protected _id: string;
    protected _showColumnsDropdown: boolean;
    protected get themeInstance(): DefaultTheme;
    protected _pageSize: number;
    get pageSize(): number;
    set pageSize(v: number);
    protected _currentPageNumber: number;
    get currentPageNumber(): number;
    set currentPageNumber(currentPageNumber: number);
    protected _isolated: boolean;
    /**
     * Flag to determine if component should bubble inner `click` action handler
     *
     * It's useful when `models-table` is used inside the row-expand
     *
     * @default false
     */
    protected get isolated(): boolean;
    protected set isolated(isolated: boolean);
    /**
     * Order of sorting for each column. Unsorted column firstly become sorted asc, then desc, then sorting is dropped again
     */
    protected sortMap: SortMap;
    /**
     * List of properties to sort table rows
     *
     * Each value is like 'propertyName:sortDirection'
     */
    protected sortProperties: TrackedArray<string>;
    /**
     * Hash of custom functions to sort table rows
     */
    protected sortFunctions: {
        [key: string]: ColumnCustomSortFn;
    };
    protected _multipleColumnsSorting: boolean;
    protected get multipleColumnsSorting(): boolean;
    protected set multipleColumnsSorting(v: boolean);
    protected get showComponentFooter(): boolean;
    protected get showCurrentPageNumberSelect(): boolean;
    protected _useNumericPagination: boolean;
    protected get useNumericPagination(): boolean;
    protected set useNumericPagination(v: boolean);
    protected get showColumnsDropdown(): boolean;
    protected set showColumnsDropdown(v: boolean);
    get useFilteringByColumns(): boolean;
    protected get filteringIgnoreCase(): boolean;
    protected get doFilteringByHiddenColumns(): boolean;
    protected get showGlobalFilter(): boolean;
    protected get focusGlobalFilter(): boolean;
    protected get checkTextTranslations(): boolean;
    get useDataGrouping(): boolean;
    currentGroupingPropertyName: string;
    /**
     * Sort direction for grouped property values
     */
    sortByGroupedFieldDirection: SortConstants;
    /**
     * Global filter value
     */
    protected _filterString: string;
    protected get filterString(): string;
    protected set filterString(v: string);
    /**
     * @default 'row'
     */
    protected _displayGroupedValueAs: string;
    get displayGroupedValueAs(): string;
    set displayGroupedValueAs(v: string);
    get collapseNumPaginationForPagesCount(): number;
    get data(): TrackedArray<ModelsTableDataItem>;
    protected get columns(): ModelsTableColumnOptions[];
    get columnComponents(): ColumnComponents;
    /**
     * @default []
     */
    protected get columnSets(): ColumnSet[];
    /**
     * List of columns shown in the table. It's created from the {@link Core.ModelsTableArgs.columns | columns} provided to the component
     *
     * @default []
     */
    processedColumns: TrackedArray<ModelsTableColumn>;
    get groupedHeaders(): GroupedHeader[][];
    protected get showPageSize(): boolean;
    expandedItems: TrackedArray<ModelsTableDataItem>;
    protected _multipleExpand: boolean;
    protected get multipleExpand(): boolean;
    protected set multipleExpand(v: boolean);
    collapsedGroupValues: TrackedArray<any>;
    /**
     * Allow or disallow to select rows on click.
     * If `false` - no row can be selected
     *
     * @default true
     */
    protected get selectRowOnClick(): boolean;
    protected _multipleSelect: boolean;
    /**
     * Allow or disallow to select multiple rows.
     * If `false` - only one row may be selected in the same time
     *
     * @default false
     */
    get multipleSelect(): boolean;
    set multipleSelect(v: boolean);
    get expandedRowComponent(): ComponentLike | null;
    get groupingRowComponent(): ComponentLike | null;
    get groupSummaryRowComponent(): ComponentLike | null;
    get groupHeaderCellComponent(): ComponentLike | null;
    selectedItems: TrackedArray<ModelsTableDataItem>;
    /**
     * List of the currently visible columns
     *
     * @default []
     */
    get visibleProcessedColumns(): ModelsTableColumn[];
    /**
     * True if all processedColumns are hidden by `isHidden`
     *
     * @default false
     */
    get allColumnsAreHidden(): boolean;
    get dataGroupProperties(): DataGroupProperty[];
    /**
     * @default []
     */
    protected get dataGroupOptions(): SelectOption[];
    /**
     * `true` if some value is set to the global filter
     *
     * @default false
     */
    get globalFilterUsed(): boolean;
    /**
     * `true` if global filter or filter by any column is used
     *
     * @default false
     */
    get anyFilterUsed(): boolean;
    /**
     * `true` if all processedColumns don't use filtering and sorting
     *
     * @default false
     */
    protected get noHeaderFilteringAndSorting(): boolean;
    /**
     * Number of pages
     *
     * @default 0
     */
    get pagesCount(): number;
    /**
     * {@link Core.ModelsTableArgs.data | data} filtered with a global filter and columns filters
     *
     * Filtering by columns is ignored if {@link Core.ModelsTableArgs.useFilteringByColumns | useFilteringByColumns} is set to `false`
     *
     * @default []
     */
    protected get filteredContent(): TrackedArray<ModelsTableDataItem>;
    /**
     * {@link filteredContent} sorted by needed properties
     *
     * @default []
     */
    protected get arrangedContent(): TrackedArray<ModelsTableDataItem>;
    /**
     * {@link filteredContent} grouped by {@link Core.ModelsTableArgs.currentGroupingPropertyName | currentGroupingPropertyName} sorted by needed properties
     *
     * @default []
     */
    get groupedArrangedContent(): TrackedArray<ModelsTableDataItem>;
    /**
     * Content of the current table page
     *
     * {@link arrangedContent} sliced for currently shown page
     *
     * @default []
     */
    get visibleContent(): TrackedArray<ModelsTableDataItem>;
    /**
     * Content of the current table page when rows grouping is used
     *
     * {@link groupedVisibleContent} sliced for currently shown page
     *
     * @default []
     */
    get groupedVisibleContent(): TrackedArray<ModelsTableDataItem[]>;
    /**
     * List of grouped property values in order to show groups in the table
     *
     * @default []
     */
    get groupedVisibleContentValuesOrder(): string[];
    /**
     * Is user on the last page
     */
    get isLastPage(): boolean;
    /**
     * Alias to `arrangedContent.length`.  Faster than retrieving arrangedContent
     * as it avoids sorting the data before calculating length.
     *
     * @default 0
     */
    get arrangedContentLength(): number;
    /**
     * Index of the first currently shown record
     *
     * @default 0
     */
    get firstIndex(): number;
    /**
     * Index of the last currently shown record
     *
     * @default 10
     */
    get lastIndex(): number;
    /**
     * List of possible {@link pageSize} values. Used to change size of `visibleContent`
     *
     * @default [10, 25, 50]
     */
    get pageSizeValues(): number[];
    /**
     * List of options for pageSize-selectBox
     * It's mapped from {@link pageSizeValues}
     *
     * @default []
     */
    pageSizeOptions: TrackedArray<SelectOption>;
    /**
     * List of options for pageNumber-selectBox
     *
     * @default []
     */
    get currentPageNumberOptions(): SelectOption[];
    /**
     * These are options for the columns dropdown.
     * By default, the 'Show All', 'Hide All' and 'Restore Defaults' buttons are displayed.
     */
    get columnDropdownOptions(): ColumnDropdownOptions;
    constructor(owner: unknown, args: ModelsTableArgs & T);
    protected didUpdateNode(): void;
    protected didInsertNode(element: HTMLElement): void;
    /**
     * @method _checkColumnTitles
     */
    protected _checkColumnTitles(): void;
    /**
     * Component init
     *
     * Set visibility and filtering attributes for each column
     *
     * @method setup
     */
    protected setup(): void;
    protected _setupArgs(): void;
    /**
     * Focus on 'Global filter' on component render
     *
     * @method focus
     */
    protected focus(element: HTMLElement): void;
    /**
     * Preselect table rows if `selectedItems` is provided
     *
     * `multipleSelected` may be set `true` if `selectedItems` has more than 1 item
     *
     * @method _setupSelectedRows
     */
    protected _setupSelectedRows(): void;
    /**
     * Generate hash for column-`extend`
     *
     * @method _createColumnInstance
     */
    protected _createColumnInstance(options: ModelsTableColumnOptions): ModelsTableColumn;
    /**
     * Create a column.
     * This can be overwritten if you need to use your own column object.
     *
     * Override must be something like:
     *
     * @example
     * ```js
     * _createColumn(options) {
     *   const column = this._createColumnInstance(options);
     *   setProperties(column, options);
     *   return column;
     * }
     * ```
     */
    protected _createColumn(options: ModelsTableColumnOptions): ModelsTableColumn;
    /**
     * Create new properties for `columns`
     */
    protected _setupColumns(): void;
    /**
     * Provide backward compatibility with `pageSizeValues` equal to an array with numbers and not objects
     * `pageSizeValues` is live as is, `pageSizeOptions` is used in the templates
     */
    protected _setupPageSizeOptions(): void;
    /**
     * Set `sortProperties` when single-column sorting is used
     */
    protected _singleColumnSorting(column: ModelsTableColumn, sortedBy?: string | number, newSorting?: SortConstants): void;
    /**
     * Set `sortProperties` when multi-columns sorting is used
     */
    protected _multiColumnsSorting(column: ModelsTableColumn, sortedBy?: string | number, newSorting?: SortConstants): void;
    /**
     * Send {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}-action when user does sort of filter.
     * Action is sent if {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged} is a closure-action
     */
    protected userInteractionObserver(overrides?: {
        [key: string]: any;
    }): void;
    /**
     * Send {@link Core.ModelsTableArgs.onColumnsVisibilityChanged | onColumnsVisibilityChanged}-action when user changes which columns are visible.
     * Action is sent if {@link Core.ModelsTableArgs.onColumnsVisibilityChanged | onColumnsVisibilityChanged} is a closure action
     */
    protected _onColumnsVisibilityChanged(): void;
    /**
     * Handler for global filter and filter by each column
     */
    protected forceToFirstPage(): void;
    /**
     * Collapse open rows when user change page size or moved to the another page
     */
    protected collapseRowOnNavigate(): void;
    /**
     * Update colspans for table header cells
     */
    protected updateHeaderCellsColspan(): void;
    protected updateHeaderCellsColspanOnce(): void;
    /**
     * Clear all filters
     */
    protected _clearFilters(): void;
    /**
     * Toggle visibility for provided column
     *
     * It does nothing if column can't be hidden (see {@link Core.ModelsTableColumn.mayBeHidden | mayBeHidden}).
     * May trigger sending {@link Core.ModelsTableArgs.onColumnsVisibilityChanged | onColumnsVisibilityChanged}
     *
     * @event toggleColumnVisibility
     */
    toggleColumnVisibility(column: ModelsTableColumn): void;
    /**
     * Show all columns
     *
     * Set each column `isHidden` value to `false`.
     * May trigger sending {@link Core.ModelsTableArgs.onColumnsVisibilityChanged | onColumnsVisibilityChanged}
     *
     * @event showAllColumns
     */
    showAllColumns(): void;
    /**
     * Hide all columns that may be hidden (see {@link Core.ModelsTableColumn.mayBeHidden | mayBeHidden})
     *
     * May trigger sending {@link Core.ModelsTableArgs.onColumnsVisibilityChanged | onColumnsVisibilityChanged}
     *
     * @event hideAllColumns
     */
    hideAllColumns(): void;
    /**
     * Restore columns visibility values according to their default visibility settings (see [defaultVisible](Utils.Core.ModelsTableColumn.html#property_defaultVisible))
     *
     * May trigger sending {@link Core.ModelsTableArgs.onColumnsVisibilityChanged | onColumnsVisibilityChanged}
     *
     * @event restoreDefaultVisibility
     */
    restoreDefaultVisibility(): void;
    /**
     * Toggle visibility for every column in the selected columns set
     *
     * It ignores columns that can't be hidden (see {@link Core.ModelsTableColumn.mayBeHidden | mayBeHidden}).
     * May trigger sending {@link Core.ModelsTableArgs.onColumnsVisibilityChanged | onColumnsVisibilityChanged}
     *
     * @event toggleColumnSetVisibility
     */
    toggleColumnSetVisibility(columnSetToToggle?: ColumnSet): void;
    /**
     * Pagination click-handler
     *
     * It moves user to the selected page.
     * Check {@link DefaultTheme.PaginationNumeric} and {@link DefaultTheme.PaginationSimple} for usage examples.
     * May trigger sending {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}
     *
     * @event goToPage
     */
    goToPage(pageNumber: number): void;
    /**
     * Sort selected column by [sortedBy](Utils.Core.ModelsTableColumn.html#property_sortedBy) or {@link Core.ModelsTableColumn.propertyName | propertyName}
     *
     * It will drop sorting for other columns if {@link multipleColumnsSorting} is set to `false`.
     * It will add new sorting if {@link multipleColumnsSorting} is set to `true`.
     * May trigger sending {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}.
     * Table will be dropped to the first page if sorting is done
     *
     * For multiColumns-sorting calling sort will change sort-order. E.g.:
     *
     * @example
     * ```js
     * sortProperties = ['a:asc', 'b:asc', 'c:desc'];
     * sort({propertyName: 'b'}); // sortProperties now is ['a:asc', 'c:desc', 'b:desc']
     * ```
     *
     * @event sort
     */
    sort(column: ModelsTableColumn): void;
    /**
     * Expand selected row
     *
     * It will cause expandedRowComponent to be used for it. It will collapse already expanded row if {@link multipleExpand} is set to `false`.
     * Expanding is assigned to the record itself and not their index.
     * So, if page #1 has first row expanded and user is moved to any another page, first row on new page won't be expanded.
     * However, when user will be back to the first page, first row will be expanded. May trigger sending {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}
     *
     * @event expandRow
     */
    expandRow(index: number, dataItem: ModelsTableDataItem): void;
    /**
     * Collapse selected row
     *
     * May trigger sending {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}
     *
     * @event collapseRow
     */
    collapseRow(index: number, dataItem: ModelsTableDataItem): void;
    /**
     * Expand all rows in the current page
     *
     * It works only if {@link multipleExpand} is set to `true`. May trigger sending {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}
     *
     * @event expandAllRows
     */
    expandAllRows(): void;
    /**
     * Collapse all rows in the current page
     *
     * May trigger sending {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}
     *
     * @event collapseAllRows
     */
    collapseAllRows(): void;
    /**
     * Handler for row-click
     *
     * Toggle `selected`-state for row. Select only one or multiple rows depends on {@link multipleSelect} value.
     * May trigger sending {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}
     *
     * @event clickOnRow
     */
    clickOnRow(index: number, dataItem: ModelsTableDataItem): void;
    /**
     * Handler for double-click on row
     *
     * May trigger sending {@link Core.ModelsTableArgs.onRowDoubleClick | onRowDoubleClick}
     *
     * @event doubleClickOnRow
     */
    doubleClickOnRow(index: number, dataItem: ModelsTableDataItem): void;
    /**
     * Handler for row-hover
     *
     * May trigger sending {@link Core.ModelsTableArgs.onRowHover | onRowHover}
     *
     * @event hoverOnRow
     */
    hoverOnRow(index: number, dataItem: ModelsTableDataItem): void;
    /**
     * Handler for row-hover
     *
     * May trigger sending {@link Core.ModelsTableArgs.onRowOut | onRowOut}
     *
     * @event outRow
     */
    outRow(index: number, dataItem: ModelsTableDataItem): void;
    /**
     * Clear all column filters and global filter
     *
     * May trigger sending {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}
     *
     * @event clearFilters
     */
    clearFilters(): void;
    /**
     * Select/deselect all rows
     *
     * May trigger sending {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}
     *
     * @event toggleAllSelection
     */
    toggleAllSelection(): void;
    /**
     * Expand or collapse all rows in the rows group
     *
     * **IMPORTANT** {@link Core.ModelsTableArgs.multipleExpand | multipleExpand} should be set to `true` otherwise this action won't do anything
     *
     * @event toggleGroupedRowsExpands
     */
    toggleGroupedRowsExpands(groupedValue: string): void;
    /**
     * Select/deselect rows from the rows group
     *
     * **IMPORTANT** {@link Core.ModelsTableArgs.multipleSelect | multipleSelect} should be set to `true` otherwise this action won't do anything
     *
     * May trigger sending {@link Core.ModelsTableArgs.onDisplayDataChanged | onDisplayDataChanged}
     *
     * @event toggleGroupedRowsSelection
     */
    toggleGroupedRowsSelection(groupedValue: string): void;
    /**
     * Collapse or expand rows group
     *
     * @event toggleGroupedRows
     */
    toggleGroupedRows(groupedValue: string): void;
    /**
     * Change number of shown rows
     *
     * @event changePageSize
     */
    changePageSize(newPageSize: string): void;
    /**
     * Change global filter value
     *
     * @event changeGlobalFilter
     */
    changeGlobalFilter(e: Event | string): void;
    /**
     * Change column filter value
     *
     * @event changeColumnFilter
     */
    changeColumnFilter(newColumnFilterValue: string, column: ModelsTableColumn): void;
    /**
     * @event changeGroupingPropertyName
     */
    changeGroupingPropertyName(newGroupingPropertyName: string): void;
    protected onClick(e?: Event): void;
}
//# sourceMappingURL=models-table.d.ts.map